export declare class WebGLState {
    noCache: boolean;
    readonly _gl: WebGLRenderingContext | WebGL2RenderingContext;
    private _dict;
    private _currentFramebuffer;
    private _currentTextureBinding;
    private _currentRenderBuffer;
    private _activeTextureIndex;
    private readonly _maxTextureCount;
    private _pixelStorei;
    _textureUnitDict: {
        [key: string]: WebGLTexture | null;
    };
    bindingVAO?: WebGLVertexArrayObject;
    constructor(gl: WebGLRenderingContext | WebGL2RenderingContext);
    destroy(): void;
    private _reset;
    toggle(capability: GLenum, enable?: boolean): void;
    /**
     * 对于该上下文开启某种特性
     * @param capability
     * example:
     * gl.enable(gl.DITHER);
     */
    enable(capability: GLenum): void;
    /**
     * 基于某种上下文关闭特性
     * @param capability
     * example:
     * gl.disable(gl.DITHER);
     */
    disable(capability: GLenum): void;
    /**
     * 绑定framebuffer webgl2新增: gl.DRAW_FRAMEBUFFER 和 gl.READ_FRAMEBUFFER
     * @param target
     * @param framebuffer
     * example:
     * const framebuffer = gl.createFramebuffer();
     * gl.bindFramebuffer(gl.FRAMEBUFFER, framebuffer);
     */
    bindFramebuffer(target: GLenum, framebuffer: WebGLFramebuffer | null): void;
    bindRenderBuffer(target: GLenum, renderBuffer: WebGLRenderbuffer | null): void;
    /**
     * 绑定系统framebuffer
     */
    bindSystemFramebuffer(): void;
    /**
     * 将定义好的WebGLProgram 对象添加到当前的渲染状态中。
     * @param program
     * example:
     * gl.useProgram(program);
     * gl.useProgram(null);
     */
    useProgram(program: WebGLProgram | null): void;
    /**
     * 使用预设值来清空缓冲
     * @param mask
     * example:
     * gl.clear(gl.DEPTH_BUFFER_BIT);
     * gl.clear(gl.DEPTH_BUFFER_BIT | gl.COLOR_BUFFER_BIT);
     */
    clear(mask: number): void;
    /**
     * 设置深度缓冲区的深度清除值
     * @param depth
     * example:
     * gl.clearDepth(0.5);
     */
    clearDepth(depth: GLclampf): void;
    /**
     * 指定将输入像素深度与当前深度缓冲区值进行比较的函数。
     * @param func
     * example:
     * gl.enable(gl.DEPTH_TEST);
     * gl.depthFunc(gl.NEVER);
     */
    depthFunc(func: GLenum): void;
    /**
     * 设置是否启用写入深度缓冲。
     * @param flag
     * example:
     * gl.depthMask(false);
     */
    depthMask(flag: boolean): void;
    polygonOffset(factor: number, unit: number): void;
    /**
     * 将 z 值从规范化设备坐标映射到窗口坐标
     * @param zNear
     * @param zFar
     * example:
     * gl.depthRange(0.2, 0.6);
     */
    depthRange(zNear: number, zFar: number): void;
    /**
     * 模版测试设置函数和引用值。
     * @param func
     * @param ref
     * @param mask
     * example:
     * gl.enable(gl.STENCIL_TEST);
     * gl.stencilFunc(gl.LESS, 0, 0b1110011);
     */
    clearStencil(s: GLint): void;
    /**
     * 控制启用和禁用模板平面中单个位的正面和背面写入
     * @param mask
     * example:
     * gl.stencilMask(0xff);
     */
    stencilMask(mask: number): void;
    /**
     * 模版测试设置函数和引用值。
     * @param func
     * @param ref
     * @param mask
     * example:
     * gl.enable(gl.STENCIL_TEST);
     * gl.stencilFunc(gl.LESS, 0, 0b1110011);
     */
    stencilFunc(func: GLenum, ref: GLint, mask: GLuint): void;
    /**
     * 单面模版测试
     * @param face
     * @param func
     * @param ref
     * @param mask
     * example:
     * gl.enable(gl.STENCIL_TEST);
     * gl.stencilFuncSeparate(gl.FRONT, gl.LESS, 0.2, 1110011);
     */
    stencilFuncSeparate(face: GLenum, func: GLenum, ref: GLint, mask: GLuint): void;
    /**
     * 单面的mask写入
     * @param face
     * @param mask
     * example:
     * gl.stencilMaskSeparate(gl.FRONT, 110101);
     */
    stencilMaskSeparate(face: GLenum, mask: GLuint): void;
    /**
     * 设置正面和背面模板测试操作
     * @param fail
     * @param zfail
     * @param zpass
     * example:
     * gl.enable(gl.STENCIL_TEST);
     * gl.stencilOp(gl.INCR, gl.DECR, gl.INVERT);
     */
    stencilOp(fail: GLenum, zfail: GLenum, zpass: GLenum): void;
    /**
     * 设置正面和/或背面模板测试操作
     * @param face
     * @param fail
     * @param zfail
     * @param zpass
     * example:
     * gl.enable(gl.STENCIL_TEST);
     * gl.stencilOpSeparate(gl.FRONT, gl.INCR, gl.DECR, gl.INVERT);
     */
    stencilOpSeparate(face: GLenum, fail: GLenum, zfail: GLenum, zpass: GLenum): void;
    /**
     * 剔除方式
     * @param mode
     * example:
     * gl.enable(gl.CULL_FACE);
     * gl.cullFace(gl.FRONT_AND_BACK);
     */
    cullFace(mode: GLenum): void;
    /**
     * 设置卷绕方向
     * @param mode
     * example:
     * gl.frontFace(gl.CW);
     */
    frontFace(mode: GLenum): void;
    /**
     * 设置颜色写入
     * @param red
     * @param green
     * @param blue
     * @param alpha
     * example:
     * gl.colorMask(true, true, true, false);
     */
    clearColor(red: GLclampf, green: GLclampf, blue: GLclampf, alpha: GLclampf): void;
    /**
     * 设置颜色写入
     * @param red
     * @param green
     * @param blue
     * @param alpha
     * example:
     * gl.colorMask(true, true, true, false);
     */
    colorMask(red: boolean, green: boolean, blue: boolean, alpha: boolean): void;
    /**
     * 设置源和目标混合因子
     * @param red
     * @param green
     * @param blue
     * @param alpha
     * example:
     * gl.blendColor(0, 0.5, 1, 1);
     */
    blendColor(red: GLclampf, green: GLclampf, blue: GLclampf, alpha: GLclampf): void;
    /**
     * 用于混合像素算法
     * @param sfactor
     * @param dfactor
     * example:
     * gl.enable(gl.BLEND);
     * gl.blendFunc(gl.SRC_COLOR, gl.DST_COLOR);
     */
    blendFunc(sfactor: GLenum, dfactor: GLenum): void;
    /**
     * 分别设置应用在 RGB 和 Alpha 上的 factor
     * @param srcRGB
     * @param dstRGB
     * @param srcAlpha
     * @param dstAlpha
     * example:
     * gl.enable(gl.BLEND);
     * gl.blendFuncSeparate(gl.SRC_COLOR, gl.DST_COLOR, gl.ONE, gl.ZERO);
     */
    blendFuncSeparate(srcRGB: GLenum, dstRGB: GLenum, srcAlpha: GLenum, dstAlpha: GLenum): void;
    /**
     * 设置混合模式
     * @param mode
     * example:
     * gl.blendEquation(gl.FUNC_ADD);
     * gl.blendEquation(gl.FUNC_SUBTRACT);
     * gl.blendEquation(gl.FUNC_REVERSE_SUBTRACT);
     */
    blendEquation(mode: GLenum): void;
    /**
     * 可以分别对 RGB 和 Alpha 做不同的操作处理
     * @param modeRGB
     * @param modeAlpha
     * example:
     * gl.blendEquationSeparate(gl.FUNC_ADD, gl.FUNC_SUBTRACT);
     */
    blendEquationSeparate(modeRGB: GLenum, modeAlpha: GLenum): void;
    /**
     * 图像预处理
     * @param pname
     * @param param
     * example:
     * var tex = gl.createTexture();
     * gl.bindTexture(gl.TEXTURE_2D, tex);
     * gl.pixelStorei(gl.PACK_ALIGNMENT, 4);
     */
    pixelStorei(pname: GLenum, param: GLenum): void;
    /**
     * 用来设置视口，即指定从标准设备到窗口坐标的x、y仿射变换。
     * @param x
     * @param y
     * @param width
     * @param height
     * example:
     * gl.viewport(0, 0, width, height);
     */
    viewport(x: number, y: number, width: number, height: number): void;
    /**
     * 激活指定的纹理单元
     * @param texture
     * example:
     * gl.activeTexture(gl.TEXTURE1);
     */
    activeTexture(texture: GLenum): void;
    /**
     * 绑定WebGLTexture
     * @param target
     * @param texture
     * @param force
     * example:
     * var texture = gl.createTexture();
     * gl.bindTexture(gl.TEXTURE_2D, texture)
     */
    bindTexture(target: GLenum, texture: WebGLTexture | null, force?: boolean): void;
    private _set1;
    private _set2;
    private _set3;
    private _set4;
    get(name: string): any;
}
