import type { GPUBuffer } from './GPUBuffer';
import type { IGPUResource, IGPURenderer } from './IGPURenderer';
import type { Immutable, TypedArray } from './type';
import type { GPURenderer } from './Renderer';
export interface GeometryBufferLayout {
    index: number;
    name: string;
}
/**
 * Attribute binding information,
 * size:attribute size. the size attribute of vec4|vec3|vec2 is 4|3|2
 * stride:used for interleaved attributes, 2 vec4 stride is 4 * 2 * Float32Array.BYTES_PER_ELEMENTS
 * offset: offset for interleaved attributes, 1 vec4 offset is 4 * Float32Array.BYTES_PER_ELEMENTS
 * normalize: attributed should be normalized
 * location: attribute location
 * Attribute的绑定信息
 * size：向量的大小，vec4｜vec3｜vec2的大小分别为 4｜3｜2
 * stride：如果使用交错存储顶点，2个vec4的stride为 2 * 4 * Float32Array.BYTES_PER_ELEMENTS
 * offset：交错顶点的数据偏移，1个vec4的偏移为 4*Float32Array.BYTES_PER_ELEMENTS
 * location: attribute location 可以通过GL反射获得，Metal必须填写
 */
export interface AttributeBase {
    size: number;
    /**
     * default is float
     */
    type?: WebGLRenderingContext['FLOAT'] | WebGLRenderingContext['INT'] | WebGLRenderingContext['SHORT'];
    stride?: number;
    offset?: number;
    instanceDivisor?: number;
    normalize?: boolean;
    location?: number;
}
/**
 * if data provided, type can be inferred from its data
 * if data not provided, type will be FLOAT by default
 * releasable is false by default,
 * if releasable set true, geometry.getAttributeData will return null after data send to gpu buffer
 * 如果有data字段，type会根据数据进行推断
 * 如果没有data字段，type默认为FLOAT
 * releasable默认为false，如果为true，在数据被提交到gpu buffer后geometry.getAttributeData方法返回null
 */
export interface AttributeWithData extends AttributeBase {
    data?: TypedArray;
    releasable?: boolean;
}
export interface AttributeWithType extends AttributeBase {
    /**
       * 如果使用interleaved attribute，此字段表示数据共享的attribute
       */
    dataSource: string;
}
export type AttributeWithDataSource = AttributeWithType;
export type Attribute = AttributeWithData | AttributeWithType | AttributeWithDataSource;
export type GeometryDrawMode = WebGLRenderingContext['POINTS'] | WebGLRenderingContext['TRIANGLES'] | WebGLRenderingContext['TRIANGLE_STRIP'] | WebGLRenderingContext['TRIANGLE_FAN'] | WebGLRenderingContext['LINES'] | WebGLRenderingContext['LINE_STRIP'] | WebGLRenderingContext['LINE_LOOP'];
export interface GeometryOptions {
    name?: string;
    attributes: {
        [key: string]: Attribute;
    };
    bufferLayouts?: GeometryBufferLayout[];
    index?: {
        data: GeometryIndexData;
        releasable?: boolean;
    };
    mode?: GeometryDrawMode;
    drawCount?: number;
    drawStart?: number;
    bufferUsage?: GLenum;
}
/**
 * 不兼容Uint8的Index，只能使用Uin16和Uin32
 */
export type GeometryIndexData = Uint16Array | Uint32Array;
export declare class Geometry implements IGPUResource {
    readonly renderer?: IGPURenderer;
    /**
       * 集合体的attribute信息，只读
       */
    readonly attributes?: Immutable<{
        [key: string]: Attribute;
    }>;
    readonly isDestroyed: boolean;
    readonly options?: Immutable<GeometryOptions>;
    indexDataType?: WebGLRenderingContext['UNSIGNED_INT'] | WebGLRenderingContext['UNSIGNED_SHORT'];
    drawStart: number;
    drawCount: number;
    mode: number;
    constructor(options: GeometryOptions, renderer?: IGPURenderer);
    assignRenderer(renderer: GPURenderer): Geometry;
    destroy(): void;
    setAttributeBuffer(name: string, buffer: GPUBuffer): void;
    getAttributeBuffer(name: string): GPUBuffer | undefined;
    /**
       * Attribute的实际数据尺寸，如果没有配置stride，则通过attribute size和attribute数据类型进行计算
       * @param name
       */
    getAttributeStride(name: string): number;
    setAttributeData(name: string, data: TypedArray): void;
    getAttributeData(name: string): TypedArray | undefined;
    setAttributeSubData(name: string, offset: number, data: TypedArray): void;
    getIndexData(): GeometryIndexData | undefined;
    setIndexData(data: GeometryIndexData | undefined): void;
    setIndexSubData(offset: number, data: TypedArray): void;
    /**
       * 获取Attribute的数据内存长度
       * 如果此Attribute设置了其他dataSource则返回 0
       * 无论是否设置了attribute releasable，此函数总是返回实际数据长度
       * @param name
       */
    getAttributeDataLength(name: string): number;
    getAttributeNames(): string[];
    /**
       * 数据写入的指令被缓存到JS中，在调用此函数后，所有JS数据指令开始调用对应的GPU函数
       * 如果geometry对象没有进行GPU对象绑定，此函数不会成功
       */
    flush(): boolean;
}
export interface SharedGeometryOptions {
    name?: string;
    drawStart?: number;
    drawCount?: number;
    mode?: number;
    index?: {
        data: GeometryIndexData;
        releasable?: boolean;
    };
    geometry: Geometry;
}
/**
 * 重用Geometry的vao对象，可以进行indexBuffer的修改
 */
export declare class SharedGeometry extends Geometry {
    /**
       * clone geometry with same gpu buffers,
       * if geometry renderer not assigned,this method returns void
       * @param options
       * @param renderer
       */
    constructor(options: SharedGeometryOptions, renderer?: IGPURenderer);
}
