import type { ShaderUseCacheId, ShaderWithSource } from './ShaderLibrary';
import type { IGPUResource, IGPURenderer } from './IGPURenderer';
import type { Immutable, TypedArray } from './type';
import type { GPURenderer } from './Renderer';
import type { DestroyOptions } from './constants';
import type { Texture } from './Texture';
import type { SemanticFunc } from './RenderFrame';
export type UniformSemantic = 'VIEW' | 'MODEL' | 'MODELVIEW' | 'PROJECTION' | 'VIEWPROJECTION' | 'MODELVIEWPROJECTION' | string;
export interface MaterialRenderStates {
    sampleAlphaToCoverage?: boolean;
    blending?: boolean;
    blendEquationRGB?: number;
    blendEquationAlpha?: number;
    blendDstAlpha?: number;
    blendSrcAlpha?: number;
    blendSrc?: number;
    blendDst?: number;
    /**
       * The blend color is a state value,
       * like the blend equations and blend parameters. Therefore,
       * it cannot change within a single draw call;
       * fragment shaders cannot write to, modify,
       * or even access the blend color.
       * All buffers share the same blend color.
       */
    blendColor?: [r: number, g: number, b: number, a: number];
    colorMask?: [r: boolean, g: boolean, b: boolean, a: boolean];
    depthTest?: boolean;
    depthMask?: boolean;
    depthFunc?: number;
    depthRange?: [zNear: number, zFar: number];
    cullFace?: number;
    cullFaceEnabled?: boolean;
    frontFace?: number;
    stencilTest?: boolean;
    stencilMask?: number;
    stencilMaskBack?: number;
    stencilMaskFront?: number;
    stencilFunc?: [func: GLenum, ref: GLint, mask: GLuint];
    stencilFuncBack?: [func: GLenum, ref: GLint, mask: GLuint];
    stencilFuncFront?: [func: GLenum, ref: GLint, mask: GLuint];
    stencilOp?: [fail: GLenum, zfail: GLenum, zpass: GLenum];
    stencilOpBack?: [fail: GLenum, zfail: GLenum, zpass: GLenum];
    stencilOpFront?: [fail: GLenum, zfail: GLenum, zpass: GLenum];
    polygonOffset?: [factor: number, units: number];
    polygonOffsetFill?: boolean;
}
export declare const enum MaterialRenderType {
    normal = 0
}
export interface MaterialOptions {
    states: MaterialRenderStates;
    shader: ShaderWithSource | ShaderUseCacheId;
    name?: string;
    dataBlocks?: MaterialDataBlockOptions[];
}
/**
 * @example
 * const mtl0 = new Material({
 *  shader:{
 *   fragment:'...',
 *   vertex:'...',
 *   shaderCacheId:'mtl_01'
 * }});
 * console.log(mtl.shaderCacheId) // expect to be a none empty string
 */
/**
 * @example
 * const mtl0 = new Material({
 *  shader:{
 *   fragment:'...',
 *   vertex:'...',
 *   cacheId:'mtl_01'
 * }});
 *
 * const mtl2 = new Material({
 *   shader:{
 *     cacheId:'mtl_01'
 *   }
 * });
 * mtl2 use the same program with mtl0
 */
export type UniformValueDataType = TypedArray | number | number[] | number[][];
export type UniformValue = UniformValueDataType | UniformStruct | UniformStruct[];
export type UniformStruct = {
    [key: string]: UniformValueDataType;
};
export interface MaterialDestroyOptions {
    textures?: DestroyOptions;
    blocks?: DestroyOptions;
}
export declare class Material implements IGPUResource {
    readonly renderer?: IGPURenderer;
    readonly name: string;
    readonly defaultDataBlock: MaterialDataBlock;
    readonly dataBlocks: MaterialDataBlock[];
    readonly options: Immutable<MaterialOptions>;
    readonly shaderCacheId: string;
    readonly states: Immutable<MaterialRenderStates>;
    readonly isDestroyed: boolean;
    constructor(options: MaterialOptions);
    addDataBlock(b: MaterialDataBlock): void;
    removeDataBlock(b: MaterialDataBlock): void;
    destroy(options?: MaterialDestroyOptions): void;
    assignRenderer(renderer: GPURenderer): Material;
}
export interface MaterialDataBlockOptions {
    uniformValues?: {
        [key: string]: UniformValue;
    };
    uniformTextures?: Record<string, Texture | Texture[]>;
    name?: string;
    uniformSemantics?: {
        [key: string]: UniformSemantic | SemanticFunc;
    };
}
export interface MaterialDataBlockDestroyOptions {
    textures?: DestroyOptions;
}
export declare class MaterialDataBlock implements IGPUResource {
    readonly renderer?: IGPURenderer;
    readonly isDestroyed: boolean;
    readonly name: string;
    readonly uniformSemantics?: {
        [key: string]: UniformSemantic | SemanticFunc;
    };
    constructor(options: MaterialDataBlockOptions);
    hasUniformValue(name: string): boolean;
    setUniformSemantic(name: string, semantic: string | SemanticFunc | undefined): void;
    /**
     * 多次调用次函数只会更新JS层的数据，不会直接更新GPU数据
     * 在下一次渲染前，会将JS层的数据更新到GPU
     * @param name
     * @param value
     */
    setUniformValue(name: string, value: UniformValue): boolean;
    /**
     * 在JS层维持一份uniform数据，当JS数据发生变化时，数据并不会直接反应到GPU
     * 需要调用setUniformValue更新数据到GPU
     * 多次调用getUniformValue返回同一个Value对象
     * @param name
     */
    getUniformValue(name: string): UniformValue;
    getUniformValues(): {
        [key: string]: UniformValue;
    };
    setUniformValues(map: {
        [key: string]: UniformValue;
    }): void;
    setUniformTexture(uniformName: string, texture: Texture | Texture[] | undefined): void;
    getUniformTexture<T extends Texture[] | Texture = Texture>(uniformName: string): T | undefined;
    destroy(options?: MaterialDataBlockDestroyOptions): void;
    assignRenderer(renderer: GPURenderer): MaterialDataBlock;
    clone(name: string): MaterialDataBlock;
}
