import type { Texture, TextureConfigOptions, TextureFormatOptions } from './Texture';
import type { Mesh, MeshDestroyOptions } from './Mesh';
import type { vec4 } from './type';
import type { IGPURenderer, IGPUResource } from './IGPURenderer';
import type { GPURenderer } from './Renderer';
import type { UniformValue } from './Material';
import type { GPUBuffer } from './GPUBuffer';
import type { RenderState, SemanticFunc } from './RenderFrame';
import type { DestroyOptions } from './constants';
export declare enum RenderPassDestroyAttachmentType {
    force = 0,
    keep = 1,
    keepExternal = 2,
    destroy = 0
}
export declare const RenderPassPriorityPrepare = 0;
export declare const RenderPassPriorityNormal = 1000;
export declare const RenderPassPriorityPostprocess = 3000;
export declare enum TextureLoadAction {
    whatever = 0,
    load = 1,
    clear = 2
}
export declare enum TextureStoreAction {
    store = 1,
    clear = 0
}
export declare enum RenderPassAttachmentStorageType {
    none = 0,
    color = 1,
    stencil_8_opaque = 2,
    depth_16_opaque = 3,
    depth_stencil_opaque = 4,
    depth_16_texture = 5,
    depth_24_stencil_8_texture = 6
}
export interface RenderPassStorageObject {
    readonly attachment: GLenum;
    readonly format: GLenum;
    readonly size: [width: number, height: number];
    readonly storageType: RenderPassAttachmentStorageType;
    readonly multiSample: number;
}
export interface RenderPassColorAttachmentTextureOptions extends TextureFormatOptions, TextureConfigOptions {
}
export interface RenderPassColorAttachmentOptions {
    texture?: Texture | RenderPassColorAttachmentTextureOptions;
    storage?: RenderPassStorageObject;
}
export interface RenderPassColorAttachment {
    readonly storageType: RenderPassAttachmentStorageType.color;
    readonly texture: Texture;
    readonly isDestroyed: boolean;
}
export interface RenderPassDepthStencilAttachment {
    readonly storageType: RenderPassAttachmentStorageType;
    readonly texture?: Texture;
    readonly storage: RenderPassStorageObject;
}
export interface RenderPassDepthStencilAttachmentOptions {
    storageType: RenderPassAttachmentStorageType;
    storage?: RenderPassStorageObject;
    texture?: Texture;
}
export interface RenderPassStoreAction {
    depthAction?: TextureStoreAction;
    stencilAction?: TextureStoreAction;
    colorAction?: TextureStoreAction;
}
export interface RenderPassLoadAction {
    clearColor?: vec4;
    colorAction?: TextureLoadAction;
    clearDepth?: number;
    depthAction?: TextureLoadAction;
    clearStencil?: number;
    stencilAction?: TextureLoadAction;
}
export interface RenderPassAttachmentOptions {
    attachments?: RenderPassColorAttachmentOptions[];
    depthStencilAttachment?: RenderPassDepthStencilAttachmentOptions;
    viewport?: [x: number, y: number, width: number, height: number];
    viewportScale?: number;
    multiSample?: number;
}
export type SemanticGetter = UniformValue | SemanticFunc | GPUBuffer | Texture | Texture[];
export interface RenderPassOptions extends RenderPassAttachmentOptions {
    name?: string;
    meshes?: Mesh[];
    priority?: number;
    meshOrder?: RenderPassMeshOrder;
    loadAction?: RenderPassLoadAction;
    storeAction?: RenderPassStoreAction;
    semantics?: {
        [key: string]: SemanticGetter;
    };
    delegate?: RenderPassDelegate;
}
export declare enum RenderPassMeshOrder {
    none = 1,
    ascending = 2,
    descending = 3
}
export type RenderPassDestroyOptions = {
    meshes?: MeshDestroyOptions | DestroyOptions.keep;
    depthStencilAttachment?: RenderPassDestroyAttachmentType;
    colorAttachment?: RenderPassDestroyAttachmentType;
    semantics?: DestroyOptions;
};
export declare class SemanticMap {
    constructor(options?: {
        [key: string]: SemanticGetter;
    });
    setSemantic(name: string, value: SemanticGetter | undefined): void;
    getSemanticValue(name: string, state: RenderState): UniformValue | GPUBuffer | null | undefined;
    hasSemanticValue(name: string): boolean;
    toObject(): Record<string, SemanticGetter>;
    destroy(): void;
}
export interface RenderPassDelegate {
    willBeginRenderPass?: (renderPass: RenderPass, state: RenderState) => void;
    didEndRenderPass?: (renderPass: RenderPass, state: RenderState) => void;
    willRenderMesh?: (mesh: Mesh, state: RenderState) => void;
    didRenderMesh?: (mesh: Mesh, state: RenderState) => void;
}
export declare class RenderPass implements IGPUResource {
    readonly isDestroyed: boolean;
    readonly name: string;
    readonly priority: number;
    readonly attachments: RenderPassColorAttachment[];
    readonly depthAttachment?: RenderPassDepthStencilAttachment;
    readonly stencilAttachment?: RenderPassDepthStencilAttachment;
    readonly viewport: [x: number, y: number, width: number, height: number];
    readonly meshes: Mesh[];
    readonly renderer?: IGPURenderer;
    readonly semantics: SemanticMap;
    readonly storeAction: RenderPassStoreAction;
    readonly loadAction: RenderPassLoadAction;
    readonly multisample: number;
    delegate: RenderPassDelegate;
    meshOrder: RenderPassMeshOrder;
    constructor(options: RenderPassOptions, renderer?: IGPURenderer);
    /**
     * reset color/depth_stencil attachments,
     * if color/depth_stencil texture is external, this will not destroy textures
     * if color/depth_stencil texture is internal created, this will also delete those textures
     * this method will delete previous fbo if exists
     * returns previous options
     **/
    resetAttachments(options: RenderPassAttachmentOptions): RenderPassAttachmentOptions;
    setMeshes(meshes: Mesh[]): Mesh[];
    addMesh(mesh: Mesh): void;
    removeMesh(mesh: Mesh): void;
    destroy(options?: RenderPassDestroyOptions): void;
    assignRenderer(renderer: GPURenderer): RenderPass;
}
