export type Shader = ShaderWithSource | ShaderUseCacheId;
export type ShaderWithSource = InstancedShaderWithSource | SharedShaderWithSource;
export interface MetalShaderSpec {
    mtlBinary?: ArrayBuffer | string;
    mtlSource?: string;
    mtlVertex?: string;
    mtlFragment?: string;
}
export interface ShaderSourceBase extends MetalShaderSpec {
    fragment: string;
    vertex: string;
    name?: string;
    downgrade?: boolean;
    marcos?: ShaderMarcos;
}
export interface InstancedShaderWithSource extends ShaderSourceBase {
    shared?: false;
}
export interface SharedShaderWithSource extends ShaderSourceBase {
    cacheId?: string;
    shared: true;
}
export interface ShaderUseCacheId {
    cacheId: string;
}
export type ShaderMarcos = [key: string, value: string | number | boolean][];
export declare enum ShaderCompileResultStatus {
    noShader = 0,
    success = 1,
    fail = 2,
    compiling = 3
}
export interface ShaderCompileResult {
    status: ShaderCompileResultStatus;
    cacheId?: string;
    error?: string | null;
    compileTime?: number;
    shared?: boolean;
}
/**
 * @example
 * const lib:ShaderLibrary = ...;
 * const shaderCacheId = lib.addShader({fragment:'...',vertex:'...'});
 * lib.compileShader(shaderCacheId);
 * //pre-compiled program
 * const mtl = new Material({
 *   shader:{ shaderCacheId }
 * })
 */
export interface ShaderLibrary {
    readonly cacheIds: string[];
    getShaderResult(cacheId: string): ShaderCompileResult | undefined;
    /**
       * 添加shader
       * @param shader
       */
    addShader(shader: Shader): string;
    /**
       * @param cacheId
       */
    deleteShader(cacheId: string): void;
    /**
       * 编译shader
       * @param shaderCacheId
       * @param asyncCallback 如果传入，则会启用异步编译，当所有编译完成后被回调
       */
    compileShader(shaderCacheId: string, asyncCallback?: (result: ShaderCompileResult) => void): ShaderCompileResult;
    /***
       * 编译现有的所有shader
       * @param asyncCallback 如果传入，则会启用异步编译，当所有编译完成后被回调
       */
    compileAllShaders(asyncCallback?: (results: ShaderCompileResult[]) => void): void;
    destroy(): void;
}
export declare class ShaderLibraryEmpty implements ShaderLibrary {
    get cacheIds(): never[];
    getShaderResult(cacheId: string): ShaderCompileResult | undefined;
    addShader(shader: Shader, marcos?: ShaderMarcos): string;
    compileAllShaders(asyncCallback?: (results: ShaderCompileResult[]) => void): never[];
    deleteShader(cacheId: string): void;
    compileShader(shaderCacheId: string): ShaderCompileResult;
    destroy(): void;
}
