import type { ShaderLibrary } from './ShaderLibrary';
import type { GPUCapability } from './GPUCapability';
import type { IGPURenderer } from './IGPURenderer';
import type { Canvas } from './Canvas';
import type { Texture } from './Texture';
import type { RenderPass } from './RenderPass';
import type { RendererInternal } from './native';

export interface RendererOptions {
  gl?: WebGLRenderingContext | WebGL2RenderingContext,
  internal?: RendererInternal,
  canvas?: HTMLCanvasElement,
  // if use frameworks [ 'webgl2', 'webgl' ], renderer will choose webgl if webgl2 is not supported
  frameworks?: Array<'webgl' | 'webgl2'>,
  willCaptureImage?: boolean,
  premultiplyAlpha?: boolean,
}

/**
 * 常用的GPU方法，不是规范必须实现的
 */
export interface RendererExtensions {
  copyTexture?: (source: Texture, tex: Texture) => void,
  resetColorAttachments?: (rp: RenderPass, colorAttachments: Texture[]) => void,
}
export declare class GPURenderer implements IGPURenderer {
  readonly height: number;
  readonly width: number;
  /**
     * if webgl not support, gpu.level will be 0
     */
  readonly shaderLibrary: ShaderLibrary;
  readonly gpu: GPUCapability;
  readonly canvas: Canvas;

  readonly extension: RendererExtensions;
  readonly isDestroyed: boolean;

  constructor (options: RendererOptions);

  /**
     * 根据不同平台封装实现 requestAnimationFrame，
     * FrameRequestCallback函数回调中进行渲染数据更新，
     * OpenGL/WebGL在绘制开始之前(传递此帧数据）
     * Metal在commandBuffer提交后（传递下一帧数据）
     * @param cb
     */
  requestAnimationFrame (cb: FrameRequestCallback): number;

  //todo
  cancelAnimationFrame (id: number): void;

  /**
     * only resize when width or height is different from now,
     * when resized,all full-screen renderPasses attached to this renderer should be resized as well
     * 只有当宽高不同的时候，才实际进行函数
     * 当重新调整宽高后，这个renderer创建的renderPasses，如果没有被指定 viewport（全屏），那么需要resize 所有的renderPass
     * @param width
     * @param height
     */
  resize (width: number, height: number): void;

  /**
     * destroy all resources created by this renderer
     * @param haltGL if true, this method will call webgl lose context extension which leads webgl absolutely halt
     */
  destroy (haltGL?: boolean): void;

  /**
   * 此帧需要进行绘制
   */
  invalid (): void;

  /**
   * 用于恢复渲染的接口
   */
  resume (): void;
}
