export type Shader = ShaderWithSource | ShaderUseCacheId;
export type ShaderWithSource = InstancedShaderWithSource | SharedShaderWithSource;
export interface MetalShaderSpec {
  mtlBinary?: ArrayBuffer | string,
  mtlSource?: string,
  mtlVertex?: string,
  mtlFragment?: string,
}

export interface ShaderSourceBase extends MetalShaderSpec {
  fragment: string,
  vertex: string,
  name?: string,
  //将es300的shader in/out关键字替换为attribute/varying
  downgrade?: boolean,
  marcos?: ShaderMarcos,
}

export interface InstancedShaderWithSource extends ShaderSourceBase {
  shared?: false,
}

export interface SharedShaderWithSource extends ShaderSourceBase {
  //相同cacheId的shader会使用同一个GLProgram
  //提供了cacheId就会认为是shared Shader
  cacheId?: string,
  //shared为true时，
  //如果没有提供cacheId，会根据字符串hash计算出 cacheId，
  //字符串相同的shader将会使用同一个GLProgram
  //如果提供了cacheId，cacheId相同的，都会使用同一个GLProgram
  shared: true,
}

export interface ShaderUseCacheId {
  cacheId: string,
}

export type ShaderMarcos = [key: string, value: string | number | boolean][];

export enum ShaderCompileResultStatus {
  noShader = 0,
  success = 1,
  fail = 2,
  compiling = 3
}

export interface ShaderCompileResult {
  status: ShaderCompileResultStatus,
  cacheId?: string,
  error?: string | null,
  compileTime?: number,
  // deprecated
  shared?: boolean,
}

/**
 * @example
 * const lib:ShaderLibrary = ...;
 * const shaderCacheId = lib.addShader({fragment:'...',vertex:'...'});
 * lib.compileShader(shaderCacheId);
 * //pre-compiled program
 * const mtl = new Material({
 *   shader:{ shaderCacheId }
 * })
 */
export interface ShaderLibrary {

  readonly cacheIds: string[],

  getShaderResult(cacheId: string): ShaderCompileResult | undefined,

  /**
     * 添加shader
     * @param shader
     */
  addShader(shader: Shader): string,

  /**
     * @param cacheId
     */
  deleteShader(cacheId: string): void,

  /**
     * 编译shader
     * @param shaderCacheId
     * @param asyncCallback 如果传入，则会启用异步编译，当所有编译完成后被回调
     */
  compileShader(shaderCacheId: string, asyncCallback?: (result: ShaderCompileResult) => void): ShaderCompileResult,

  /***
     * 编译现有的所有shader
     * @param asyncCallback 如果传入，则会启用异步编译，当所有编译完成后被回调
     */
  compileAllShaders(asyncCallback?: (results: ShaderCompileResult[]) => void): void,

  destroy(): void,
}

export class ShaderLibraryEmpty implements ShaderLibrary {

  get cacheIds () {
    return [];
  }

  getShaderResult (cacheId: string): ShaderCompileResult | undefined {
    return undefined;
  }

  addShader (shader: Shader, marcos?: ShaderMarcos) {
    return '';
  }

  compileAllShaders (asyncCallback?: (results: ShaderCompileResult[]) => void) {
    return [];
  }

  deleteShader (cacheId: string) {
  }

  compileShader (shaderCacheId: string): ShaderCompileResult {
    return { status: ShaderCompileResultStatus.fail };
  }

  destroy () {
  }
}
