import type { IGPURenderer, IGPUResource } from './IGPURenderer';
import type { Immutable, TypedArray } from './type';
import type { GPURenderer } from './Renderer';

/**
 * 图片解码的格式，默认为 WebGL2RenderingContext['RGBA8']
 * 如果图片解码格式和texture的不一致，在非Web环境下，会产生报错，无法上传数据
 * 一般情况由系统根据图片文件解析出图片的像素格式
 */
export type ImageBitmapPixelFormat = WebGL2RenderingContext['RGBA8'] |
WebGL2RenderingContext['RGBA16F'] |
WebGL2RenderingContext['RGBA32F'];

export enum TextureSourceType {
  none,
  data = 1,
  image = 2,
  compressed = 3,
  video = 4,
  canvas = 5,
  framebuffer = 6,
  mipmaps = 7
}

export interface TextureFactorySource2DFrom {
  type: TextureSourceType.image | TextureSourceType.video,
  url: string,
  target?: WebGLRenderingContext['TEXTURE_2D'],
}

export interface TextureFactorySourceFromCompressed {
  type: TextureSourceType.compressed,
  url: string,
  c?: number,
}

export type TextureFactorySourceCubeFrom = {
  type: TextureSourceType.image,
  map: TextureCubeSourceURLMap | string[],
  target: WebGLRenderingContext['TEXTURE_CUBE_MAP'],
};

type DataRange = [start: number, length: number];

export type TextureFactorySourceCubeBinaryMipmapsFrom = {
  target: WebGLRenderingContext['TEXTURE_CUBE_MAP'],
  type: TextureSourceType.mipmaps,
  bin: string,
  mipmaps: Array<[
        TEXTURE_CUBE_MAP_POSITIVE_X: DataRange,
        TEXTURE_CUBE_MAP_NEGATIVE_X: DataRange,
        TEXTURE_CUBE_MAP_POSITIVE_Y: DataRange,
        TEXTURE_CUBE_MAP_NEGATIVE_Y: DataRange,
        TEXTURE_CUBE_MAP_POSITIVE_Z: DataRange,
        TEXTURE_CUBE_MAP_NEGATIVE_Z: DataRange
  ]>,
};

export type TextureFactorySource2DBinaryMipmapsFrom = {
  target?: WebGLRenderingContext['TEXTURE_2D'],
  type: TextureSourceType.mipmaps,
  bin: string,
  mipmaps: Array<DataRange>,
};

export interface TextureFactorySourceCubeMipmapsFrom {
  type: TextureSourceType.mipmaps,
  target: WebGLRenderingContext['TEXTURE_CUBE_MAP'],
  maps: TextureCubeSourceURLMap[] | string[][],
}

export interface TextureFactorySource2DMipmapsFrom {
  type: TextureSourceType.mipmaps,
  target?: WebGLRenderingContext['TEXTURE_2D'],
  urls: string[],
}

export type TextureFactorySourceFrom =
    TextureFactorySource2DBinaryMipmapsFrom
    | TextureFactorySourceCubeBinaryMipmapsFrom
    | TextureFactorySource2DFrom
    | TextureFactorySourceCubeFrom
    | TextureFactorySourceCubeMipmapsFrom
    | TextureFactorySource2DMipmapsFrom
    | TextureFactorySourceFromCompressed;

export interface TextureFactory {
  /**
     * 通过TextureConfigOptions可以决定图片解码时使用的格式
     * @param options
     * @param config texture的属性
     */
  loadSourceAsync(options: TextureFactorySourceFrom, config?: TextureConfigOptions): Promise<TextureOptions>,

  canOffloadTexture(texture: Texture): boolean,

  reloadTextureAsync(texture: Texture): Promise<Texture>,

  loadCompressedTextureFromArrayBuffer(arrayBuffer: ArrayBuffer, options?: TextureOptions): TextureOptions,
}

export declare class Texture implements IGPUResource {
  readonly width: number;
  readonly height: number;
  readonly id: string;
  readonly options: Immutable<TextureOptions>;
  readonly renderer?: IGPURenderer;
  readonly sourceType: TextureSourceType;
  readonly name: string;
  readonly isDestroyed: boolean;

  constructor (options: TextureOptions, renderer?: IGPURenderer);

  destroy (): void;

  updateSource (options: TextureSourceOptions): void;

  uploadCurrentVideoFrame (): boolean;

  offloadData (): void;

  reloadDataAsync (): Promise<Texture>;

  assignRenderer (renderer: GPURenderer): Texture;

}

type ImageSource = ImageBitmap | HTMLImageElement | HTMLCanvasElement;

export interface ImageData {
  data: TypedArray,
  width: number,
  height: number,
}

export type TextureSourceCubeData = [
    TEXTURE_CUBE_MAP_POSITIVE_X: ImageSource | ImageData,
    TEXTURE_CUBE_MAP_NEGATIVE_X: ImageSource | ImageData,
    TEXTURE_CUBE_MAP_POSITIVE_Y: ImageSource | ImageData,
    TEXTURE_CUBE_MAP_NEGATIVE_Y: ImageSource | ImageData,
    TEXTURE_CUBE_MAP_POSITIVE_Z: ImageSource | ImageData,
    TEXTURE_CUBE_MAP_NEGATIVE_Z: ImageSource | ImageData
];

// 为减少内存消耗，除了video之外，其余的数据内容会在texture生成后被置为undefined
// const tex = new Texture({image:myImage},renderer)
// tex.options.image === undefined

export type TextureSourceOptions = Texture2DSourceOptions | TextureCubeSourceOptions;

export interface Texture2DSourceOptionsNone extends TextureOptionsBase {
  sourceType: TextureSourceType.none,
  target?: GLenum,
}

export interface Texture2DSourceOptionsImage extends TextureOptionsBase {
  sourceType: TextureSourceType.image,
  image: ImageSource,
  target?: WebGLRenderingContext['TEXTURE_2D'],
  generateMipmap?: boolean,
}

export interface Texture2DSourceOptionsFrameBuffer extends TextureOptionsBase {
  sourceType: TextureSourceType.framebuffer,
  data?: { width: number, height: number },
  target?: WebGLRenderingContext['TEXTURE_2D'],
}

export interface Texture2DSourceOptionsData extends TextureOptionsBase {
  sourceType: TextureSourceType.data,
  data: { data: TypedArray, width: number, height: number },
  target?: WebGLRenderingContext['TEXTURE_2D'],
}

export interface Texture2DSourceOptionsVideo extends TextureOptionsBase {
  sourceType: TextureSourceType.video,
  video: HTMLVideoElement,
  target?: WebGLRenderingContext['TEXTURE_2D'],
  generateMipmap?: boolean,
}

export interface Texture2DSourceOptionsImageMipmaps extends TextureOptionsBase {
  sourceType: TextureSourceType.mipmaps,
  mipmaps: Array<ImageSource | { data: TypedArray, width: number, height: number }>,
  target?: WebGLRenderingContext['TEXTURE_2D'],
}

export interface Texture2DSourceOptionsCompressed extends TextureOptionsBase {
  sourceType: TextureSourceType.compressed,
  mipmaps: Array<{ data: Uint8Array, width: number, height: number }>,
  target?: WebGLRenderingContext['TEXTURE_2D'],
}

export interface TextureCubeSourceOptionsImage extends TextureOptionsBase {
  sourceType: TextureSourceType.image | TextureSourceType.data,
  target: WebGLRenderingContext['TEXTURE_CUBE_MAP'],
  cube: TextureSourceCubeData,
  generateMipmap?: boolean,
}

export interface TextureCubeSourceOptionsImageMipmaps extends TextureOptionsBase {
  sourceType: TextureSourceType.mipmaps,
  mipmaps: Array<TextureSourceCubeData>,
  target: WebGLRenderingContext['TEXTURE_CUBE_MAP'],
}

export type TextureCubeSourceURLMap = [
    TEXTURE_CUBE_MAP_POSITIVE_X: string,
    TEXTURE_CUBE_MAP_NEGATIVE_X: string,
    TEXTURE_CUBE_MAP_POSITIVE_Y: string,
    TEXTURE_CUBE_MAP_NEGATIVE_Y: string,
    TEXTURE_CUBE_MAP_POSITIVE_Z: string,
    TEXTURE_CUBE_MAP_NEGATIVE_Z: string
];

export interface TextureConfigOptionsBase {
  name?: string,
  wrapS?: GLenum, // GL_CLAMP_TO_EDGE
  wrapT?: GLenum, // GL_CLAMP_TO_EDGE
  magFilter?: GLenum, // GL_NEAREST
  minFilter?: GLenum, // GL_NEAREST
  anisotropic?: number, //0
  flipY?: boolean,
  premultiplyAlpha?: boolean,
  //when use ImageBitMap, the bitmap content will be closed after sent to GPU
  keepImageSource?: boolean,
}

export interface TextureConfigOptions extends TextureConfigOptionsBase {
  sourceFrom?: TextureFactorySourceFrom,
  premultiplyAlpha?: boolean,
}

export interface TextureFormatOptions {
  format?: GLenum, // GL_RGBA
  internalFormat?: GLenum, // GL_RGBA
  type?: GLenum, //GL_UNSIGNED_BYTE
}

export interface TextureOptionsBase extends TextureConfigOptions, TextureFormatOptions {
  sourceType: TextureSourceType,
}

export type TextureCubeSourceOptions = TextureCubeSourceOptionsImage | TextureCubeSourceOptionsImageMipmaps;

export type Texture2DSourceOptions =
    Texture2DSourceOptionsImage
    | Texture2DSourceOptionsData
    | Texture2DSourceOptionsVideo
    | Texture2DSourceOptionsImageMipmaps
    | Texture2DSourceOptionsCompressed
    | Texture2DSourceOptionsFrameBuffer;

export type Texture2DOptions = Texture2DSourceOptions;

export type TextureCubeOptions = TextureCubeSourceOptions;

export type TextureOptions = Texture2DOptions | TextureCubeOptions;
