import type { ImageBitmapCallback, ImageBitmapConstructor } from './ImageBitmapInternal';
import type { TypedArray, vec2 } from '../type';
import type { GPUCapability } from '../GPUCapability';
import type { PredyResizeTemplate, SDFImageOptions, SDFImage } from '@predy-js/specification';
import type { ImageBitmapInternal } from './ImageBitmapInternal';
import type { PredyVideoDecoder, PredyVideoDecoderConstructor } from './VideoDecoder';
import type { ResourcePlatform } from './ResourceInternal';
import type { PredyVideoCodec, PredyTextEncoding, PredyResourceCacheStatus } from '../constants';

export interface PredyRequestOptions {
  url: string,
  //用于测试缓存，生产中不要使用
  disableCache?: boolean,
  //默认5S
  timeout?: number,

  responseType?: 'text' | 'arraybuffer',

  //使用系统网络库（仅用于试验）
  useSystemNetwork?: boolean,
}

export interface PredyRequestResponse {
  data: ArrayBuffer | string,
  cacheStatus: PredyResourceCacheStatus,
  time: number,
}

export type PredyRequestCallback = (err: string | undefined, res: PredyRequestResponse) => void;

export const DataCompressMethodZlib = 1;

type DataCompressMethod = typeof DataCompressMethodZlib;

export interface PredyNativeInternal {
  webpDisabled?: boolean,
  /**
     * 是否支持获取gles3，但如果获取es3失败，会降级到es2
     */
  supportGLES3?: boolean,
  platform: ResourcePlatform,
  createImageBitmap(data: TypedArray | string, //图片文件的数据
    options: ImageBitmapConstructor, callback: ImageBitmapCallback): void,
  getDefaultGPUCapability(): GPUCapability,
  requestWithCache(options: PredyRequestOptions, callback: PredyRequestCallback): void,
  /**
     * decode text from data
     * @param data
     * @param encoding 默认utf8
     */
  decodeText(data: Uint8Array, encoding?: PredyTextEncoding): string,
  /**
     * 将图片绘制到画布上，同时替换文案，绘制的顺序如下
     * 创建template.size尺寸的画布, 接下来的绘制均应用 template.offset
     * 将images绘制到画布上，images的尺寸由image.frame决定
     * 将数据模板content绘制到画布上,使用variables替换文案
     * @param template
     */
  renderImageTemplate(template: PredyResizeTemplate): ImageBitmapInternal,
  /**
     * 异步绘制图片的模板
     * @param options
     * @param callback 如果传入callback就是异步绘制，否则会直接返回结果
     */
  generateSDFImage(options: SDFImageOptions, callback?: renderSDFImageCallback): SDFImage | undefined,
  /**
     * 下载并且执行脚本文件，此方法会缓存https的脚本内容
     * 对于http的请求（开发环境），每次都重新请求
     * @param url
     * @param callback
     */
  evaluateScriptURL(url: string, callback: (err: Error, result: any) => void): void,
  /**
     * 将float32Array转换为half float，写入Uint16Array中,
     * 如果提供了range参数，代表float16会被重新map到range中，以此提高精度范围
     * 提供range参数后，算法求出source的最大最小值，然后将float16重新映射到range中
     * 如果source和target的长度不匹配，返回false
     */
  float32ToFloat16(source: Float32Array, target: Uint16Array, range?: vec2): boolean,

  /**
     * 将half float转换为float32，通过Float32Array返回，
     * 如果提供了range参数，代表float16会被重新map到range中，以此提高精度范围
     * 如果没有提供，float16会被直接转化为float32
     * 如果source和target的长度不匹配，返回false
     */
  float16ToFloat32(source: Uint16Array, target: Float32Array, range?: vec2): boolean,

  /**
     * 解压数据
     * @param data
     * @param method
     */
  inflateData(data: Uint8Array, method: DataCompressMethod): Uint8Array | undefined,

  /**
     * 恢复当前的主JSContext,多个JSContext执行的时候会发生抢占，
     * 调用此函数后，当前的JSContext会被恢复
     */
  restoreJSContext(): boolean,

  /**
     * 创建 video decoder
     * @param options
     */
  createVideoDecoder(options: PredyVideoDecoderConstructor): PredyVideoDecoder,

  /**
     * 是否支持视频编码
     * @param codec
     */
  supportVideoCodec(codec: PredyVideoCodec): boolean,
}

type renderSDFImageCallback = (img: SDFImage) => void;
