import type { ResourceInternal, ResourcePlatform } from './ResourceInternal';
import type { GPUBufferInternal, GPUBufferInternalOptions } from './GPUBufferInternal';
import type { GeometryInternal, GeometryInternalOptions } from './GeometryInternal';
import type { ShaderLibraryInternal } from './ShaderLibraryInternal';
import type { DataBlockInternal, DataBlockInternalOptions } from './DataBlockInternal';
import type { MaterialInternal, MaterialInternalOptions } from './MaterialInternal';
import type { MeshInternal, MeshInternalOptions } from './MeshInternal';
import type { RenderPassInternal, RenderPassOptionsInternal } from './RenderPassInternal';
import type { RenderFrameInternal, RenderFrameInternalOptions } from './RenderFrameInternal';
import type { GPUCapability } from '../GPUCapability';
import type { TextureInternal, TextureInternalConstructOptions } from './TextureInternal';
import type { AndGLContext } from './AndGLContext';
import type { MetalInternal } from './MetalInternal';
import type { vec2 } from '../type';

/**
 * renderer 内部创建的对象，用来统计信息，排查资源泄漏的问题
 */
export interface InternalResInfo {
  buffer: number,
  geometry: number,
  material: number,
  texture: number,
  dataBlock: number,
  mesh: number,
  renderPass: number,
  frame: number,
}

export interface RendererInternalCompleteInfo {
  //gpu耗时，单位ms
  gpuTime?: number,
  //渲染了多少个frame
  frame: number,
  //cpu耗时
  cpuTime?: number,
}

export enum RendererLostReason {
  unknown = 0,
  removeByParent = 1,
  glContextLost = 2,
}

export abstract class RendererInternal implements ResourceInternal {
  readonly isDestroyed: boolean;
  readonly platform: ResourcePlatform;
  readonly gpu: GPUCapability;
  readonly shaderLibrary: ShaderLibraryInternal;
  readonly width: number;
  readonly height: number;
  /**
   * 用于Android的GL上下文, 用于测试， 不会在RI中使用
   */
  readonly gl?: AndGLContext;

  //todo lilong
  /**
   * metal相关设置
   */
  readonly metal?: MetalInternal;

  abstract destroy (): void;

  /**
     * 需要进行重新渲染
     */
  abstract invalid (): void;

  abstract createBuffer (options: GPUBufferInternalOptions): GPUBufferInternal;

  abstract createGeometry (options: GeometryInternalOptions): GeometryInternal;

  abstract createDataBlock (options: DataBlockInternalOptions): DataBlockInternal;

  abstract createMaterial (options: MaterialInternalOptions): MaterialInternal;

  abstract createMesh (options: MeshInternalOptions): MeshInternal;

  abstract createRenderPass (options: RenderPassOptionsInternal): RenderPassInternal;

  abstract createRenderFrame (options: RenderFrameInternalOptions): RenderFrameInternal;

  abstract createTexture (options: TextureInternalConstructOptions): TextureInternal;

  abstract getResourceInfo (): InternalResInfo;

  abstract requestAnimationFrame (callback: FrameRequestCallback): number;

  abstract cancelAnimationFrame (id: number): void;

  /**
   * 当view被从客户端移除时，会触发此回调
   * 如果丢失GL Context也会触发此函数
   * 当此函数发生时，需要立刻销毁Renderer对象
   * 如果renderer已经被销毁了，这个回调不会被调用
   * @param callback
   */
  abstract setRendererLostCallback (callback: (reason: RendererLostReason) => void): void;

  /**
   * 下一帧渲染完成后的回调,在调用removeRenderCompleteCallback前，不需要重复注册
   */
  abstract addRenderCompleteCallback (callback: (info: RendererInternalCompleteInfo) => void): number;

  abstract removeRenderCompleteCallback (id: number): void;
  /**
   * 获取错误信息，目前支持最大获取5条，
   * 每次调用此函数后，清空错误队列
   */
  abstract getErrors (): string[];

  /**
   * resize predy view，并且相关的fbo和render pass都需要设为相应的尺寸
   * @param size 如果不提供size，则使用当前predy view parent的尺寸
   */
  abstract resize (size?: vec2): void;
}

