import type { VideoFrameFormat } from '@predy-js/specification';
import type { PredyResourceCacheStatus } from '../constants';
export interface PredyVideoDecoderFrameData {
  width: number,
  height: number,
  data: Uint8Array,
  idx: number,
  keyframe: boolean,
}

export enum PredyVideoDecoderStatus {
  init = 0, // 创建对象
  loading = 1, // 正在解析metadata，解析过程任何函数调用都会导致error
  metadataReady = 2, // metadata 解析完成，此时 currentFrame = -1
  seekingFrame = 3, // 正在解析某一帧，解析过程任何函数调用都会导致error
  frameReady = 4, // 解析某一帧完成，此时 currentFrame = frame.index
  destroyed = 5, // 对象已经销毁
  error = 44, // 发生错误
}

/**
 * 视频解码的通用class，在native和web端有不同的实现
 */
export interface PredyVideoDecoderMetadata {
  width: number,
  height: number,
  duration: number, // in ms
  frameCount: number,
  cacheStatus: PredyResourceCacheStatus,
  format: VideoFrameFormat,
  frameBufferSize: number, // 每帧的buffer大小
}

/**
 * VideoDecoder创建参数
 */
export interface PredyVideoDecoderConstructor {
  name: string,
  prefetchOnly?: boolean,
}

export type PredyVideoDecoderLoadedCallback = (error?: Error, metadata?: PredyVideoDecoderMetadata) => void;
export type PredyVideoDecoderSeekFrameCallback = (error?: Error, frame?: PredyVideoDecoderFrameData) => void;

export interface PredyVideoDecoder {
  readonly name: string,
  readonly status: PredyVideoDecoderStatus,
  readonly metadata?: PredyVideoDecoderMetadata,

  /**
     * 只做资源请求，不进行视频解码，
     * 这种情况下，seekFrame永远返回error（prefetch only）
     */
  readonly prefetchOnly: boolean,
  /**
     * 是否要跳播/回播，如果不需要的话，Decoder可以自行释放已经播放的帧
     */
  readonly willReverseTime: boolean,
  readonly currentFrame: number,

  /**
     * 加载视频,解析视频的metadata，设置frame数组,
     * 此函数只能调用一次，如果需要解析其他的视频，需要重新创建对象
     * @param url 视频地址
     * @param callback 解析回调
     */
  loadVideo(url: string, callback: PredyVideoDecoderLoadedCallback): void,

  /**
     * 如果当前正在解析，中断解析，并且之前的callback返回error（abort）
     * 调用后回到frameReady状态
     */
  abortSeeking(): void,

  /**
     * 解析视频的下一帧，获得图像数据。如果调用的时候正在解析，callback直接返回error
     * @param frameIndex frame的index
     * @param data 图像数据buffer，数据将被写入到此Buffer中，如果buffer尺寸不对，解析会失败
     * @param callback 图像数据回调
     */
  seekFrame(frameIndex: number, data: Uint8Array, callback: PredyVideoDecoderSeekFrameCallback): void,

  /**
     * 释放资源
     */
  destroy(): void,
}
